#include <stdio.h>
#include <time.h>
#include <string.h>    //strlen
#include <sys/socket.h>
#include <arpa/inet.h> //inet_addr
#include <unistd.h>    //write
#include <iostream>
#include <cstdlib>
#include "common.h"

void usage()
{
    debug("Usage:\n");
    debug("device_simulator [port]\n");
}

main(int argc , char *argv[])
{
    if (argc != 2)
    {
        debug("Invalid number of arguments.\n");
	usage();
        return 1;
    }

    int port = atoi(argv[1]);

    debug_n("Port: %d.", port);

    //Create socket
    int socket_desc = socket(AF_INET , SOCK_STREAM , 0);
    if (socket_desc == -1)
    {
        debug("Socket creation failed with error: %s\n", strerror(errno));
        return 1;
    }
    debug("Socket created.\n");

    if (persistent_bind(socket_desc, port))
    {
        usage();
        return 2;
    }

    //Listen
    listen(socket_desc , 3);

    //Accept and incoming connection
    debug_n("Waiting for incoming connections.");
    int c = sizeof(struct sockaddr_in);

    int requestsProcessed = 0;

    struct sockaddr_in client;
    //accept connection from an incoming client
    int client_sock = accept(socket_desc, (struct sockaddr *)&client, (socklen_t*)&c);
    if (client_sock < 0)
    {
        debug("accept failed with error: %s\n", strerror(errno));
        return 3;
    }
    debug("Connection accepted\n");

    start_tcp_stats_collecting(client_sock);

    for(;;)
    {

        int requestsProcessed = 0;

        // send payload
        int iterationNumber = 0;
        // initialize default response size
        int responseSize = 1024;
        char* response = (char*) malloc(sizeof(char) * responseSize);
        int messageSize = 1024;
        char* message = (char*) malloc(sizeof(char) * messageSize);
        
        struct timespec t1, t2;

        clock_gettime(CLOCK_MONOTONIC, &t1);

        int tempMessageSize = ReceiveMessage(client_sock, message, messageSize);

        if (tempMessageSize < 0)
        {
            debug_n("Received too big message.");
            return 2;
        }

        // parse the message
        int* msgPtr = (int*)message;
        int requestID = *msgPtr++;
        int executionTime = *msgPtr++;
        int tempResponseSize = *msgPtr++;

        debug_n("Message received with requesId %d, execution time %d, response size %d.", requestID, executionTime, tempResponseSize);

        usleep(executionTime);

        // populate response
        *((int*)response) = requestID;

        if (SendMessage(client_sock, response, tempResponseSize))
        {
            debug("Sending response.\n");
        }
 
        clock_gettime(CLOCK_MONOTONIC, &t2);
	

    }
    // if we are here, it means client disconnected, close socket
    close(client_sock);
    debug("Client disconnected.\n");
    return 0;
}

